/**
  ******************************************************************************
  * File Name          : es9822pro.h
  * Description        : Defines for ES9822PRO ADC
  ******************************************************************************
  */
#ifndef __ES9822PRO_H
#define __ES9822PRO_H

#include "main.h"
#include "adc.h"

/* ES9822PRO I2C */
#define ES9822PRO_I2C_ADDR					0x40 // i2c slave address, 01000xx0 = 0x4x

/* ES9822PRO GPIO */
#define ES9822PRO_MONO_PIN 					I2SIN_OUT1_PIN
#define ES9822PRO_MONO_PORT 				I2SIN_OUT1_PORT
#define ES9822PRO_ADC_ENABLE_PIN 			I2SIN_OUT2_PIN
#define ES9822PRO_ADC_ENABLE_PORT 			I2SIN_OUT2_PORT
#define ES9822PRO_MCK_ENABLE_PIN 			I2SIN_OUT3_PIN
#define ES9822PRO_MCK_ENABLE_PORT 			I2SIN_OUT3_PORT
#define ES9822PRO_CLKSEL_PIN 				I2SIN_OUT4_PIN
#define ES9822PRO_CLKSEL_PORT 				I2SIN_OUT4_PORT

/* ES9822PRO registers */
#define ES9822PRO_SYS_CONFIG					0
#define ES9822PRO_ADC_CLOCK_CONFIG1        		1
#define ES9822PRO_ADC_CLOCK_CONFIG2				2
#define ES9822PRO_ADC_CLOCK_CONFIG3				3
#define ES9822PRO_ADC_CLOCK_CONFIG4				4
#define ES9822PRO_I2S_TDM_MASTER_MODE_CONFIG	8
#define ES9822PRO_I2S_TDM_MASTER_CLK_CONFIG		9
#define ES9822PRO_TDM_CONFIG1					10
#define ES9822PRO_TDM_CONFIG2					11
#define ES9822PRO_TDM_SLOT_CONFIG_CH1			12
#define ES9822PRO_TDM_SLOT_CONFIG_CH2			13
#define ES9822PRO_RAW_DATA_MAPPING				17
#define ES9822PRO_PCM_DATA_OUTPUT_MAPPING		18
#define ES9822PRO_FS_GEN_PHASE_CONTROL			23
#define ES9822PRO_INTERRUPT						27
#define ES9822PRO_SYNC_CONTROL					34
#define ES9822PRO_I2S_DECODER_CONFIG1			59
#define ES9822PRO_I2S_DECODER_CONFIG2   	    60
#define ES9822PRO_ADC_CH1A_CONFIG1				63
#define ES9822PRO_ADC_CH1A_CONFIG2				64
#define ES9822PRO_ADC_CH2A_CONFIG1				65
#define ES9822PRO_ADC_CH2A_CONFIG2				66
#define ES9822PRO_ADC_CH1B_CONFIG1				67
#define ES9822PRO_ADC_CH1B_CONFIG2				68
#define ES9822PRO_ADC_CH2B_CONFIG1				69
#define ES9822PRO_ADC_CH2B_CONFIG2				70
#define ES9822PRO_ADC_COMMON_MODE_CONFIG		71
#define ES9822PRO_VREF_CONFIG					73
#define ES9822PRO_GPIO_1_2_CONFIG				74
#define ES9822PRO_GPIO_3_4_CONFIG				75
#define ES9822PRO_GPIO_5_6_CONFIG				76
#define ES9822PRO_GPIO_7_8_CONFIG				77
#define ES9822PRO_GPIO_9_10_CONFIG				78
#define ES9822PRO_GPIO_11_CONFIG				79
#define ES9822PRO_GPIO_SDB1						86
#define ES9822PRO_GPIO_SDB2						87
#define ES9822PRO_GPIO_OE1						88
#define ES9822PRO_GPIO_OE2						89
#define ES9822PRO_ADC_CH1_DATAPATH_CONTROL		101
#define ES9822PRO_ADC_CH1_PEAK_DETECTOR_CONFIG	105
#define ES9822PRO_ADC_CH1_PEAK_DETECTOR_LEVEL	106
#define ES9822PRO_ADC_CH1_VOLUME1           	109
#define ES9822PRO_ADC_CH1_VOLUME2				110
#define ES9822PRO_ADC_CH1_SCALE					112
#define ES9822PRO_ADC_CH1_PROG_FILTER			113
#define ES9822PRO_ADC_CH2_DATAPATH_CONTROL		118
#define ES9822PRO_ADC_CH2_PEAK_DETECTOR_CONFIG	122
#define ES9822PRO_ADC_CH2_PEAK_DETECTOR_LEVEL	123
#define ES9822PRO_ADC_CH2_VOLUME1				126
#define ES9822PRO_ADC_CH2_VOLUME2				127
#define ES9822PRO_ADC_CH2_SCALE					129
#define ES9822PRO_ADC_CH2_PROG_FILTER			130
#define ES9822PRO_ADC_CH1B_DATAPATH_CONTROL		135
#define ES9822PRO_ADC_CH3_PROG_FILTER			147
#define ES9822PRO_ADC_CH2B_DATAPATH_CONTROL		152
#define ES9822PRO_ADC_CH4_PROG_FILTER			164
#define ES9822PRO_SOFT_RESET					192
#define ES9822PRO_CLK_SELECT					193
#define ES9822PRO_ADC_CLOCK_DIVIDE				194
#define ES9822PRO_CLIP_FLAG						229
#define ES9822PRO_CH1_PEAK1						237
#define ES9822PRO_CH1_PEAK2						238
#define ES9822PRO_CH2_PEAK1						242
#define ES9822PRO_CH2_PEAK2						243

typedef enum {
	MINIMUM_PHASE						= 0,	// Minimum phase
	LINEAR_PHASE_APO					= 1,	// Linear phase apodizing
	LINEAR_PHASE_FAST					= 2,	// Linear phase fast roll off
	LINEAR_PHASE_FAST_LOW_RIPPLE		= 3,	// Linear phase fast roll off low ripple
	LINEAR_PHASE_SLOW					= 4,	// Linear phase slow roll off
	MINIMUM_PHASE_FAST					= 5,	// Minimum phase fast roll off
	MINIMUM_PHASE_SLOW					= 6,	// Minimum phase slow roll off
	MINIMUM_PHASE_SLOW_LOW_DISPERSION	= 7,	// Minimum phase slow roll off low dispersion
} ES9822PRO_FilterTypeDef;

uint8_t ES9822PRO_Init(uint8_t attenuation);
uint8_t ES9822PRO_Status();
void ES9822PRO_Reset();
uint8_t ES9822PRO_SetFs(uint32_t fs);
uint8_t ES9822PRO_THD_Comp_C2(int16_t comp);
uint8_t ES9822PRO_THD_Comp_C3(int16_t comp);
uint8_t ES9822PRO_SetMono(bool mono);
uint8_t ES9822PRO_SetFilter(uint8_t);
uint8_t ES9822PRO_SetPeakDetectorLevel(uint8_t, uint8_t);
uint8_t ES9822PRO_ClearClipDetection(uint8_t);
uint8_t ES9822PRO_ReadClipFlag(void);
void ES9822PRO_Record(void);

#endif	/* __ES9038Q2M_H */
