/**
  ******************************************************************************
  * File Name          : adc.c
  * Description        : ADC functions
  ******************************************************************************
  */
#include "main.h"
#include "adc.h"
#include "i2c.h"
#include "ak5394.h"
#include "es9822pro.h"
#include "DWT_delay.h"

TIM_HandleTypeDef htim11;

extern I2C_HandleTypeDef hi2c2;
I2C_HandleTypeDef *adc_i2c;

static bool monoState = false;

/* TIM11 init function */
static void TIM11_Init(void)
{
	htim11.Instance = TIM11;
	htim11.Init.Prescaler = SystemCoreClock / 10000 - 1; // 100 us
	htim11.Init.CounterMode = TIM_COUNTERMODE_UP;
	htim11.Init.Period = 9;
	htim11.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
	htim11.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;
	if (HAL_TIM_Base_Init(&htim11) != HAL_OK) {
		Error_Handler();
	}
}

void ADC_LowLevel_DeInit(void)
{
}

/**
  * @brief  Initializes the IOs used to drive ADC chips
  * @param  None
  * @retval None
  */
void ADC_LowLevel_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStruct = {0};

	__HAL_RCC_GPIOC_CLK_ENABLE();
	__HAL_RCC_GPIOE_CLK_ENABLE();
	__HAL_RCC_GPIOG_CLK_ENABLE();
	HAL_GPIO_WritePin(I2SIN_OUT1_PORT, I2SIN_OUT1_PIN, GPIO_PIN_SET);
	HAL_GPIO_WritePin(I2SIN_OUT2_PORT, I2SIN_OUT2_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(I2SIN_OUT3_PORT, I2SIN_OUT3_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(I2SIN_OUT4_PORT, I2SIN_OUT4_PIN, GPIO_PIN_RESET);

	/*Configure I2SIN_OUT1 pin */
	GPIO_InitStruct.Pin = I2SIN_OUT1_PIN;
	GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_InitStruct.Speed = GPIO_SPEED_LOW;
	GPIO_InitStruct.Pull = GPIO_PULLDOWN;
	HAL_GPIO_Init(I2SIN_OUT1_PORT, &GPIO_InitStruct);

	/*Configure I2SIN_OUT2 pin */
	GPIO_InitStruct.Pin = I2SIN_OUT2_PIN;
	GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(I2SIN_OUT2_PORT, &GPIO_InitStruct);

	/*Configure I2SIN_OUT3 pin */
	GPIO_InitStruct.Pin = I2SIN_OUT3_PIN;
	HAL_GPIO_Init(I2SIN_OUT3_PORT, &GPIO_InitStruct);

	/*Configure I2SIN_OUT4 pin */
	GPIO_InitStruct.Pin = I2SIN_OUT4_PIN;
	HAL_GPIO_Init(I2SIN_OUT4_PORT, &GPIO_InitStruct);

	if (_cfg.adc == ADC_ES9822PRO) {
		HAL_GPIO_WritePin(MONO_LED_PORT, MONO_LED_PIN, GPIO_PIN_RESET);
		HAL_GPIO_WritePin(CLIP_CH1_LED_PORT, CLIP_CH1_LED_PIN, GPIO_PIN_RESET);
		HAL_GPIO_WritePin(CLIP_CH2_LED_PORT, CLIP_CH2_LED_PIN, GPIO_PIN_RESET);

		/* LED indicators */
		GPIO_InitStruct.Pin = MONO_LED_PIN;
		HAL_GPIO_Init(MONO_LED_PORT, &GPIO_InitStruct);
		GPIO_InitStruct.Pin = CLIP_CH1_LED_PIN;
		HAL_GPIO_Init(CLIP_CH1_LED_PORT, &GPIO_InitStruct);
		GPIO_InitStruct.Pin = CLIP_CH2_LED_PIN;
		HAL_GPIO_Init(CLIP_CH2_LED_PORT, &GPIO_InitStruct);

		/* CLIP interrupt inputs */
		GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
		GPIO_InitStruct.Pull = GPIO_NOPULL;
		GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
		GPIO_InitStruct.Pin = I2SIN_IN1_PIN;
		HAL_GPIO_Init(I2SIN_IN1_PORT, &GPIO_InitStruct);
		GPIO_InitStruct.Pin = I2SIN_IN2_PIN;
		HAL_GPIO_Init(I2SIN_IN2_PORT, &GPIO_InitStruct);

		/* Mono switch */
		GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
		GPIO_InitStruct.Pull = GPIO_PULLUP;
		GPIO_InitStruct.Pin = MONO_SWITCH_PIN;
		HAL_GPIO_Init(MONO_SWITCH_PORT, &GPIO_InitStruct);

		/* EXTI interrupt init*/
		HAL_NVIC_SetPriority(EXTI0_IRQn, 8, 0);
		HAL_NVIC_EnableIRQ(EXTI0_IRQn);
		HAL_NVIC_SetPriority(EXTI2_IRQn, 8, 0);
		HAL_NVIC_EnableIRQ(EXTI2_IRQn);
		HAL_NVIC_SetPriority(EXTI3_IRQn, 8, 0);
		HAL_NVIC_EnableIRQ(EXTI3_IRQn);

		TIM11_Init();
	}
}

void ADC_Init(void)
{
	if (_cfg.adc == ADC_UNDEF) {
		return;
	}
	adc_i2c = &hi2c2;
	ADC_LowLevel_Init();
    if (_cfg.adc == ADC_AK5394) {
    	AK5394_Init();
    }
    else if (_cfg.adc == ADC_ES9822PRO) {
     	ES9822PRO_Init(0);
		monoState = HAL_GPIO_ReadPin(MONO_SWITCH_PORT, MONO_SWITCH_PIN);
    }
}

void ADC_Status(void)
{
	if (_cfg.adc == ADC_AK5394) {
		AK5394_Status();
	}
    else if (_cfg.adc == ADC_ES9822PRO) {
    	ES9822PRO_Status();
    }
}

void ADC_Reset(void)
{
	if (_cfg.adc == ADC_AK5394) {
		AK5394_Reset();
	}
	else if (_cfg.adc == ADC_ES9822PRO) {
		ES9822PRO_Reset();
	}
}

void ADC_SetFS(uint32_t fs)
{
	if (_cfg.adc == ADC_AK5394) {
		AK5394_SetFS(fs);
	}
	else if (_cfg.adc == ADC_ES9822PRO) {
		ES9822PRO_SetFs(fs);
	}
}

void ADC_SetMono(bool mono)
{
	if (_cfg.adc == ADC_ES9822PRO) {
		ES9822PRO_SetMono(mono);
	}
}

void ADC_Record(void)
{
	if (_cfg.adc == ADC_ES9822PRO) {
		LED_OFF(CLIP_CH1_LED_PIN, CLIP_CH1_LED_PORT);
		LED_OFF(CLIP_CH2_LED_PIN, CLIP_CH2_LED_PORT);
		ES9822PRO_Record();
	}
}

static bool monosw_debouncing = false;

bool monosw_pressed(void)
{
	bool newState = HAL_GPIO_ReadPin(MONO_SWITCH_PORT, MONO_SWITCH_PIN);
	return (monoState ? newState : !newState);
}

bool monosw_debounce(void)
{
	static uint16_t state = 1;
	state = (state << 1) | monosw_pressed() | 0xe000;
	if (state == 0xf000) {
		state = 1;
		monosw_debouncing = false;
		return true;
	}
	return false;
}

/**
  * @brief  EXTI line detection callback.
  * @param  GPIO_Pin Specifies the port pin connected to corresponding EXTI line.
  * @retval None
  */
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
	if (GPIO_Pin == MONO_SWITCH_PIN) {
		if (!monosw_debouncing) {
			monosw_debouncing = true;
			__HAL_TIM_CLEAR_FLAG(&htim11, TIM_FLAG_UPDATE);
			__HAL_TIM_SET_COUNTER(&htim11, 0L);
			HAL_TIM_Base_Start_IT(&htim11); // start debounce timer interrupt
		}
	}
	else if (GPIO_Pin == I2SIN_IN1_PIN) {
		// Channel 2 clip
		if (_cfg.debug) {
			printf("CH2 clip\r\n");
		}
		LED_ON(CLIP_CH2_LED_PIN, CLIP_CH2_LED_PORT);
	}
	else if (GPIO_Pin == I2SIN_IN2_PIN) {
		// Channel 1 clip
		if (_cfg.debug) {
			printf("CH1 clip\r\n");
		}
		LED_ON(CLIP_CH1_LED_PIN, CLIP_CH1_LED_PORT);
	}
}

/**
 * @brief  Period elapsed callback in non blocking mode
 * @param  htim : TIM handle
 * @retval None
 */
void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim)
{
	if (htim->Instance == TIM6) {
		HAL_IncTick();
	}
	else if (htim->Instance == TIM11) {
		if (monosw_debounce()) {
			__HAL_TIM_CLEAR_FLAG(htim, TIM_FLAG_UPDATE);
			HAL_TIM_Base_Stop_IT(htim);
			monoState = HAL_GPIO_ReadPin(MONO_SWITCH_PORT, MONO_SWITCH_PIN);
			ADC_SetMono(monoState);
		}
	}
	else if (htim->Instance == TIM10) {
		__HAL_TIM_CLEAR_FLAG(htim, TIM_FLAG_UPDATE);
		// AK5394 reset pulse finished
		HAL_GPIO_WritePin(AK5394_RSTN_PORT, AK5394_RSTN_PIN, GPIO_PIN_SET);
		if (_cfg.debug) {
			printf("period TIM10 %ld\r\n", HAL_GetTick());
		}
	}
}

void HAL_TIM_OC_DelayElapsedCallback(TIM_HandleTypeDef *htim)
{
	if (htim->Instance == TIM10) {
		__HAL_TIM_CLEAR_FLAG(htim, TIM_FLAG_CC1);
		// Start AK5394 reset pulse
		HAL_GPIO_WritePin(AK5394_RSTN_PORT, AK5394_RSTN_PIN, GPIO_PIN_RESET);
		if (_cfg.debug) {
			printf("oc TIM10 %ld\r\n", HAL_GetTick());
		}
	}
}
