/**
  ******************************************************************************
  * File Name          : ak5394.c
  * Description        : AK5394 functions
  ******************************************************************************
  */
#include <stdio.h>
#include "main.h"
#include "ak5394.h"

TIM_HandleTypeDef htim10;	// AK5394 reset pulse timer
static void TIM10_Init(void);

/**
  * @brief TIM10 Initialization Function
  * @param None
  * @retval None
  */
static void TIM10_Init(void)
{
	TIM_OC_InitTypeDef sConfigOC = {0};
	htim10.Instance = TIM10;
	htim10.Init.Prescaler = SystemCoreClock / 10000 - 1; // 100 us
	htim10.Init.CounterMode = TIM_COUNTERMODE_UP;
	htim10.Init.Period = AK5394_SETTLING_TIME + AK5394_RSTN_PULSE_WIDTH;
	htim10.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
	htim10.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;
	if (HAL_TIM_Base_Init(&htim10) != HAL_OK) {
		Error_Handler();
	}
	if (HAL_TIM_OC_Init(&htim10) != HAL_OK) {
		Error_Handler();
	}
	if (HAL_TIM_OnePulse_Init(&htim10, TIM_OPMODE_SINGLE) != HAL_OK) {
		Error_Handler();
	}
	sConfigOC.OCMode = TIM_OCMODE_PWM2;
	sConfigOC.Pulse = AK5394_SETTLING_TIME;
	sConfigOC.OCPolarity = TIM_OCPOLARITY_LOW;
	sConfigOC.OCFastMode = TIM_OCFAST_DISABLE;
	if (HAL_TIM_OC_ConfigChannel(&htim10, &sConfigOC, TIM_CHANNEL_1) != HAL_OK) {
		Error_Handler();
	}
//	HAL_TIM_MspPostInit(&htim10);
}

void AK5394_LowLevelInit(void)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};

	/* GPIO Ports Clock Enable */
	__HAL_RCC_GPIOC_CLK_ENABLE();
	__HAL_RCC_GPIOE_CLK_ENABLE();
	__HAL_RCC_GPIOG_CLK_ENABLE();

	/*Configure GPIO pin Output Level */
	HAL_GPIO_WritePin(AK5394_CLK_SEL_PORT, AK5394_CLK_SEL_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(AK5394_RSTN_PORT, AK5394_RSTN_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(AK5394_DFS0_PORT, AK5394_DFS0_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(AK5394_DFS1_PORT, AK5394_DFS1_PIN, GPIO_PIN_RESET);


	/*Configure GPIO pins : CLK_SEL */
	GPIO_InitStruct.Pin = AK5394_CLK_SEL_PIN;
	GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(AK5394_CLK_SEL_PORT, &GPIO_InitStruct);

	/*Configure GPIO pins : RSTN */
	GPIO_InitStruct.Pin = AK5394_RSTN_PIN;
	HAL_GPIO_Init(AK5394_RSTN_PORT, &GPIO_InitStruct);

	/*Configure GPIO pins : DFS0 */
	GPIO_InitStruct.Pin = AK5394_DFS0_PIN;
	HAL_GPIO_Init(AK5394_DFS0_PORT, &GPIO_InitStruct);

	/*Configure GPIO pins : DFS1 */
	GPIO_InitStruct.Pin = AK5394_DFS1_PIN;
	HAL_GPIO_Init(AK5394_DFS1_PORT, &GPIO_InitStruct);

	TIM10_Init();
}

void AK5394_Init(void)
{
	AK5394_LowLevelInit();

	AK5394_Reset();
	/* Initialize AK5394 pins */
	HAL_GPIO_WritePin(AK5394_CLK_SEL_PORT, AK5394_CLK_SEL_PIN, GPIO_PIN_SET);	// 192k (48k multiples)
	HAL_GPIO_WritePin(AK5394_DFS0_PORT, AK5394_DFS0_PIN, GPIO_PIN_RESET); 		// Quad Speed
	HAL_GPIO_WritePin(AK5394_DFS1_PORT, AK5394_DFS1_PIN, GPIO_PIN_SET);
}

void AK5394_Status(void)
{
}

void AK5394_Reset(void)
{
	if (_cfg.debug) {
		printf("AK5394 reset %ld\r\n", HAL_GetTick());
	}
	HAL_GPIO_WritePin(AK5394_RSTN_PORT, AK5394_RSTN_PIN, GPIO_PIN_SET);
	// Start AK5394 RSTN pulse timer (TIM10)
	__HAL_TIM_CLEAR_FLAG(&htim10, TIM_FLAG_UPDATE);
	__HAL_TIM_CLEAR_FLAG(&htim10, TIM_FLAG_CC1);
	HAL_TIM_Base_Start_IT(&htim10);
	HAL_TIM_OnePulse_Start_IT(&htim10, TIM_CHANNEL_1);
}

void AK5394_SetFS(uint32_t fs)
{
	switch (fs) {
		case 44100L:
			HAL_GPIO_WritePin(AK5394_CLK_SEL_PORT, AK5394_CLK_SEL_PIN, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(AK5394_DFS0_PORT, AK5394_DFS0_PIN, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(AK5394_DFS1_PORT, AK5394_DFS1_PIN, GPIO_PIN_RESET);
			break;
		case 48000L:
			HAL_GPIO_WritePin(AK5394_CLK_SEL_PORT, AK5394_CLK_SEL_PIN, GPIO_PIN_SET);
			HAL_GPIO_WritePin(AK5394_DFS0_PORT, AK5394_DFS0_PIN, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(AK5394_DFS1_PORT, AK5394_DFS1_PIN, GPIO_PIN_RESET);
			break;
		case 88200L:
			HAL_GPIO_WritePin(AK5394_CLK_SEL_PORT, AK5394_CLK_SEL_PIN, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(AK5394_DFS0_PORT, AK5394_DFS0_PIN, GPIO_PIN_SET);
			HAL_GPIO_WritePin(AK5394_DFS1_PORT, AK5394_DFS1_PIN, GPIO_PIN_RESET);
			break;
		case 96000L:
			HAL_GPIO_WritePin(AK5394_CLK_SEL_PORT, AK5394_CLK_SEL_PIN, GPIO_PIN_SET);
			HAL_GPIO_WritePin(AK5394_DFS0_PORT, AK5394_DFS0_PIN, GPIO_PIN_SET);
			HAL_GPIO_WritePin(AK5394_DFS1_PORT, AK5394_DFS1_PIN, GPIO_PIN_RESET);
			break;
		case 176400L:
			HAL_GPIO_WritePin(AK5394_CLK_SEL_PORT, AK5394_CLK_SEL_PIN, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(AK5394_DFS0_PORT, AK5394_DFS0_PIN, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(AK5394_DFS1_PORT, AK5394_DFS1_PIN, GPIO_PIN_SET);
			break;
		case 192000L:
			HAL_GPIO_WritePin(AK5394_CLK_SEL_PORT, AK5394_CLK_SEL_PIN, GPIO_PIN_SET);
			HAL_GPIO_WritePin(AK5394_DFS0_PORT, AK5394_DFS0_PIN, GPIO_PIN_RESET);
			HAL_GPIO_WritePin(AK5394_DFS1_PORT, AK5394_DFS1_PIN, GPIO_PIN_SET);
			break;
		default:
			break;
	}
}
