/**
  ******************************************************************************
  * File Name          : es9822pro.c
  * Description        : ES9822PRO functions
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
/* Scheduler include files. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

#include "main.h"
#include "util.h"
#include "i2c.h"
#include "adc.h"
#include "es9822pro.h"
#include "DWT_delay.h"
#include <stdio.h>
#include <string.h>
#include <stdbool.h>

uint8_t ES9822PRO_InitSyncSlaveRegisters(void)
{
	int devaddr = 0x48;
	HAL_StatusTypeDef status;
	do {
		status = I2C_WriteRegister(adc_i2c, devaddr, ES9822PRO_CLK_SELECT, 0x03);
		if (status != HAL_OK) break;
		if (_cfg.adc_clk_frequency == CLK_22M_24M) {
			status = I2C_WriteRegister(adc_i2c, devaddr, ES9822PRO_ADC_CLOCK_DIVIDE, 0x00);
		}
		else if (_cfg.adc_clk_frequency == CLK_45M_49M) {
			status = I2C_WriteRegister(adc_i2c, devaddr, ES9822PRO_ADC_CLOCK_DIVIDE, 0x01);
		}
	} while (false);
	return status;
}

uint8_t ES9822PRO_InitRegisters(void)
{
	HAL_StatusTypeDef status;
	do {
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CLOCK_CONFIG1, 0xff);
		if (_cfg.adc_clk_frequency == CLK_22M_24M) {
			status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CLOCK_CONFIG3, 0x00);
			if (status != HAL_OK) break;
		}
		else if (_cfg.adc_clk_frequency == CLK_45M_49M) {
			status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CLOCK_CONFIG3, 0x01);
			if (status != HAL_OK) break;
		}
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_I2S_TDM_MASTER_MODE_CONFIG, 0x05);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_TDM_SLOT_CONFIG_CH1, 0x20);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_TDM_SLOT_CONFIG_CH2, 0x21);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH1A_CONFIG1, 0xbb);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH1A_CONFIG2, 0x38);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH2A_CONFIG1, 0xbb);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH2A_CONFIG2, 0x38);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH1B_CONFIG1, 0xbb);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH1B_CONFIG2, 0x38);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH2B_CONFIG1, 0xbb);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH2B_CONFIG2, 0x38);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_COMMON_MODE_CONFIG, 0xff);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_GPIO_1_2_CONFIG, 0x22);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_GPIO_3_4_CONFIG, 0x20);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_GPIO_SDB1, 0x03);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_GPIO_OE1, 0x0b);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH1_DATAPATH_CONTROL, 0x00); // 0x04
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH2_DATAPATH_CONTROL, 0x00); // 0x04
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH1B_DATAPATH_CONTROL, 0x01);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH2B_DATAPATH_CONTROL, 0x01);
		if (status != HAL_OK) break;
		status = ES9822PRO_SetFilter(LINEAR_PHASE_APO);
		if (status != HAL_OK) break;
		// GPIO10 for channel 1 clip detection
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_GPIO_9_10_CONFIG, 0x40);
		if (status != HAL_OK) break;
		// GPIO11 for channel 2 clip detection
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_GPIO_11_CONFIG, 0x05);
		if (status != HAL_OK) break;
		// Enable GPIO10 & GPIO11 output
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_GPIO_OE2, 0x06);
		if (status != HAL_OK) break;
		status = ES9822PRO_SetMono(HAL_GPIO_ReadPin(MONO_SWITCH_PORT, MONO_SWITCH_PIN));
	} while (false);
	if (status != HAL_OK) {
		printf("ES9822PRO init registers failed\r\n");
	}
	return status;
}

uint8_t ES9822PRO_Init(uint8_t attenuation)
{
	HAL_GPIO_WritePin(ES9822PRO_CLKSEL_PORT, ES9822PRO_CLKSEL_PIN, GPIO_PIN_RESET); //SET = Use X2
	DWT_Delay_us(100);
	HAL_GPIO_WritePin(ES9822PRO_MCK_ENABLE_PORT, ES9822PRO_MCK_ENABLE_PIN, GPIO_PIN_SET); //MCK enable
	DWT_Delay_us(100);
	HAL_GPIO_WritePin(ES9822PRO_ADC_ENABLE_PORT, ES9822PRO_ADC_ENABLE_PIN, GPIO_PIN_SET); //ADC enable
	DWT_Delay_us(200);
	HAL_StatusTypeDef status;
	do {
		status = ES9822PRO_InitSyncSlaveRegisters();
		if (status != HAL_OK) break;
		DWT_Delay_us(500);
		status = ES9822PRO_InitRegisters();
		if (status != HAL_OK) break;
		if (_cfg.debug) {
			uint8_t value1, value2, value3, value4;
			I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH1_VOLUME1, &value1);
			I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH1_VOLUME2, &value2);
			I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH1_SCALE, &value3);
			I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_I2S_TDM_MASTER_CLK_CONFIG, &value4);
			printf("adc %02x %02x %02x %02x", value1, value2, value3, value4);
		}
	} while (false);
	if (status != HAL_OK) {
		printf("ES9822PRO init failed\r\n");
	}
	return status;
}

uint8_t ES9822PRO_Status()
{
	return 0;
}

void ES9822PRO_Reset(void)
{
	HAL_GPIO_WritePin(ES9822PRO_ADC_ENABLE_PORT, ES9822PRO_ADC_ENABLE_PIN, GPIO_PIN_RESET);
	DWT_Delay_us(5);
	HAL_GPIO_WritePin(ES9822PRO_ADC_ENABLE_PORT, ES9822PRO_ADC_ENABLE_PIN, GPIO_PIN_SET);
}

uint8_t ES9822PRO_SetFs(uint32_t fs)
{
	HAL_GPIO_WritePin(ES9822PRO_ADC_ENABLE_PORT, ES9822PRO_ADC_ENABLE_PIN, GPIO_PIN_RESET);
	DWT_Delay_us(10);
	if (fs%44100 == 0) {
		HAL_GPIO_WritePin(ES9822PRO_CLKSEL_PORT, ES9822PRO_CLKSEL_PIN, GPIO_PIN_RESET); //RESET = Use X1
	}
	else {
		HAL_GPIO_WritePin(ES9822PRO_CLKSEL_PORT, ES9822PRO_CLKSEL_PIN, GPIO_PIN_SET); //SET = Use X2
	}
	DWT_Delay_us(20);
	HAL_GPIO_WritePin(ES9822PRO_ADC_ENABLE_PORT, ES9822PRO_ADC_ENABLE_PIN, GPIO_PIN_SET);
	DWT_Delay_us(40);
	ES9822PRO_InitSyncSlaveRegisters();
	DWT_Delay_us(200);
	ES9822PRO_InitRegisters();
	uint8_t divider = 0x03;
	bool mode2x = false;
	if (_cfg.adc_clk_frequency == CLK_22M_24M) {
		switch (fs) {
			case 44100L:
			case 48000L:
				divider = 0x03;
				break;
			case 88200L:
			case 96000L:
				divider = 0x01;
				break;
			case 176400L:
			case 192000L:
				divider = 0x00;
				break;
			case 352800L:
			case 384000L:
				divider = 0x00;
				mode2x = true;
				break;
			default:
				if (_cfg.debug) {
					printf("ES9822PRO Unsupported sampling rate: %ldHz\r\n", fs);
				}
				break;
		}
	}
	else if (_cfg.adc_clk_frequency == CLK_45M_49M) {
		switch (fs) {
			case 44100L:
			case 48000L:
				divider = 0x07;
				break;
			case 88200L:
			case 96000L:
				divider = 0x03;
				break;
			case 176400L:
			case 192000L:
				divider = 0x01;
				break;
			case 352800L:
			case 384000L:
				divider = 0x00;
				break;
			case 705600L:
			case 768000L:
				divider = 0x00;
				mode2x = true;
				break;
			default:
				if (_cfg.debug) {
					printf("ES9822PRO Unsupported sampling rate: %ldHz\r\n", fs);
				}
				break;
		}
	}
	uint8_t status;
	do {
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CLOCK_CONFIG2, divider);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_I2S_TDM_MASTER_CLK_CONFIG, divider);
		if (status != HAL_OK) break;
		uint8_t value;
		status = I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_SYS_CONFIG, &value);
		if (status != HAL_OK) break;
		if (mode2x) {
			value |= 0x08;
		}
		else {
			value &= ~0x08;
		}
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_SYS_CONFIG, value);
	} while (false);
	if (status != HAL_OK) {
		printf("ES9822PRO Set FS failed\r\n");
	}
	return status;
}

uint8_t ES9822PRO_THD_Comp_C2(int16_t comp)
{
	return 0;
}

uint8_t ES9822PRO_THD_Comp_C3(int16_t comp)
{
	return 0;
}

uint8_t ES9822PRO_SetMono(bool mono)
{
	HAL_StatusTypeDef status;
	uint8_t value;
	do {
		if (mono) {
			HAL_GPIO_WritePin(ES9822PRO_MONO_PORT, ES9822PRO_MONO_PIN, GPIO_PIN_SET); //SET = mono
			status = I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_SYS_CONFIG, &value);
			if (status != HAL_OK) break;
			value |= 0x02;
			status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_SYS_CONFIG, value);
			if (status != HAL_OK) break;
			LED_ON(MONO_LED_PIN, MONO_LED_PORT);
		}
		else {
			HAL_GPIO_WritePin(ES9822PRO_MONO_PORT, ES9822PRO_MONO_PIN, GPIO_PIN_RESET);
			status = I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_SYS_CONFIG, &value);
			if (status != HAL_OK) break;
			value &= ~0x02;
			status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_SYS_CONFIG, value);
			if (status != HAL_OK) break;
			LED_OFF(MONO_LED_PIN, MONO_LED_PORT);
		}
//		if (_cfg.debug) {
			printf("ADC mode: %s\r\n", mono ? "mono" : "stereo");
//		}
	} while (false);
	if (status != HAL_OK) {
		printf("ES9822PRO set mono failed\r\n");
	}
	return status;
}

char *getFilter(uint8_t filter)
{
	char *s = "NONE";
	switch (filter) {
	case MINIMUM_PHASE:
		s = "Minimum phase";
		break;
	case LINEAR_PHASE_APO:
		s = "Linear phase apodizing";
		break;
	case LINEAR_PHASE_FAST:
		s = "Linear phase fast roll-off";
		break;
	case LINEAR_PHASE_FAST_LOW_RIPPLE:
		s = "Linear phase fast roll-off low ripple";
		break;
	case LINEAR_PHASE_SLOW:
		s = "Linear phase slow roll-off";
		break;
	case MINIMUM_PHASE_FAST:
		s = "Minimum phase fast roll-off";
		break;
	case MINIMUM_PHASE_SLOW:
		s = "Minimum phase slow roll-off";
		break;
	case MINIMUM_PHASE_SLOW_LOW_DISPERSION:
		s = "Minimum phase slow roll-off low dispersion";
		break;
	default:
		break;
	}
	return s;
}

uint8_t ES9822PRO_SetFilter(uint8_t filter)
{
	HAL_StatusTypeDef status;
	uint8_t value = 0x80 | (filter << 2);
	do {
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH1_PROG_FILTER, value);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH2_PROG_FILTER, value);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH3_PROG_FILTER, value);
		if (status != HAL_OK) break;
		status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH4_PROG_FILTER, value);
		if (_cfg.debug) {
			printf("ADC filter: %s\r\n", getFilter(filter));
		}
	} while (false);
	if (status != HAL_OK) {
		printf("ES9822PRO set filter failed\r\n");
	}
	return status;
}

uint8_t ES9822PRO_SetClipDetection(uint8_t channel)
{
	HAL_StatusTypeDef status;
	uint8_t value = 0;
	do {
		if (channel == 1) {
			status = I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_INTERRUPT, &value);
			if (status != HAL_OK) break;
			value &= ~0x10;
			value |= 0x01;
			status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_INTERRUPT, value);
		}
		else {
			status = I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_INTERRUPT, &value);
			if (status != HAL_OK) break;
			value &= ~0x20;
			value |= 0x02;
			status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_INTERRUPT, value);
		}
		if (_cfg.debug) {
			printf("ADC set peak detection CH%d\r\n", channel);
		}
	} while (false);
	if (status != HAL_OK) {
		printf("ES9822PRO set clip detection failed\r\n");
	}
	return status;
}

void ES9822PRO_MaskClipDetection(uint8_t channel)
{
	uint8_t value = 0;
	if (channel == 1) {
		I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_INTERRUPT, &value);
		value &= ~0x01;
		I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_INTERRUPT, value);
	}
	else {
		I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_INTERRUPT, &value);
		value &= ~0x02;
		I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_INTERRUPT, value);
	}
	if (_cfg.debug) {
		printf("ADC mask peak detection CH%d\r\n", channel);
	}
}

uint8_t ES9822PRO_ClearClipDetection(uint8_t channel)
{
	HAL_StatusTypeDef status;
	uint8_t value = 0;
	do {
		if (channel == 1) {
			status = I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_INTERRUPT, &value);
			if (status != HAL_OK) break;
			value |= 0x10;
			status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_INTERRUPT, value);
		}
		else {
			status = I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_INTERRUPT, &value);
			if (status != HAL_OK) break;
			value |= 0x20;
			status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_INTERRUPT, value);
		}
		if (_cfg.debug) {
			printf("ADC clear clip detection CH%d\r\n", channel);
		}
	} while (false);
	if (status != HAL_OK) {
		printf("ES9822PRO clear clip detection failed\r\n");
	}
	return status;
}

uint8_t ES9822PRO_SetPeakDetectorLevel(uint8_t channel, uint8_t level)
{
	HAL_StatusTypeDef status;
	uint8_t value = 0;
	do {
		if (channel == 1) {
			status = I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH1_PEAK_DETECTOR_CONFIG, &value);
			if (status != HAL_OK) break;
			value |= 0x01;
			status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH1_PEAK_DETECTOR_CONFIG, value);
			if (status != HAL_OK) break;
			status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH1_PEAK_DETECTOR_LEVEL, level);
		}
		else {
			status = I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH2_PEAK_DETECTOR_CONFIG, &value);
			if (status != HAL_OK) break;
			value |= 0x01;
			status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH2_PEAK_DETECTOR_CONFIG, value);
			if (status != HAL_OK) break;
			status = I2C_WriteRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_ADC_CH2_PEAK_DETECTOR_LEVEL, level);
		}
		if (_cfg.debug) {
			printf("ADC peak detector CH%d: %02x\r\n", channel, level);
		}
	} while (false);
	if (status != HAL_OK) {
		printf("ES9822PRO set peak detector level failed\r\n");
	}
	return status;
}

uint8_t ES9822PRO_ReadClipFlag(void)
{
	uint8_t value = 0;
	I2C_ReadRegister(adc_i2c, ES9822PRO_I2C_ADDR, ES9822PRO_CLIP_FLAG, &value);
	if (_cfg.debug) {
		printf("ADC clip flag %02x\r\n", value);
	}
	return value;
}

void ES9822PRO_Record(void)
{
	ES9822PRO_ClearClipDetection(1);
	ES9822PRO_ClearClipDetection(2);
	ES9822PRO_SetPeakDetectorLevel(1, 0x7f);	// -6dB
	ES9822PRO_SetPeakDetectorLevel(2, 0x7f);	// -6dB
	ES9822PRO_SetClipDetection(1);
	ES9822PRO_SetClipDetection(2);
}
