/**
  ******************************************************************************
  * File Name          : i2c.c
  * Description        : I2C functions
  ******************************************************************************
  */
#include "main.h"
#include "i2c.h"
#include "ui_task.h"
#include <stdio.h>

HAL_StatusTypeDef I2C_WriteRegister(I2C_HandleTypeDef *i2c, int devaddr, uint8_t regaddr, uint8_t value)  // write value to a data register
{
	uint8_t buf[2];
	buf[0] = regaddr;
	buf[1] = value;
	HAL_StatusTypeDef status = HAL_I2C_Master_Transmit(i2c, devaddr, buf, 2, 10);
	if (status != HAL_OK) {
#ifdef X
		ui_msg_t msg;
		sprintf(msg.buf, "I2CW %02x %3d %1d", devaddr, regaddr, status);
		msg.x = 0;
		msg.y = 1;
		xQueueSendToBackFromISR(ui_queue, &msg, 0U);
#endif
printf("I2CW %s %02x %3d %02x %08lx\r\n", ((i2c->Instance == I2C1) ? "I2C1" : "I2C2"), devaddr, regaddr, value, i2c->ErrorCode);
	}
	return status;
}

HAL_StatusTypeDef I2C_ReadRegister(I2C_HandleTypeDef *i2c, int devaddr, uint8_t regaddr, uint8_t *value)  // Read a data register value
{
	HAL_StatusTypeDef status = HAL_I2C_Master_Transmit(i2c, devaddr, &regaddr, 1, 10);
	if (status != HAL_OK) {
#ifdef X
		ui_msg_t msg;
		sprintf(msg.buf, "I2CR1 %02x %3d %1d", devaddr, regaddr, status);
		msg.x = 0;
		msg.y = 1;
		xQueueSendToBackFromISR(ui_queue, &msg, 0U);
#endif
printf("I2CR1 %s %02x %3d %08lx\r\n", ((i2c->Instance == I2C1) ? "I2C1" : "I2C2"), devaddr, regaddr, i2c->ErrorCode);
		return status;
	}
	status = HAL_I2C_Master_Receive(i2c, devaddr, value, 1, 10);
	if (status != HAL_OK) {
#ifdef X
		ui_msg_t msg;
		sprintf(msg.buf, "I2CR2 %02x %3d %1d", devaddr, regaddr, status);
		msg.x = 0;
		msg.y = 1;
		xQueueSendToBackFromISR(ui_queue, &msg, 0U);
#endif
		printf("I2CR2 %s %02x %3d %08lx\r\n", ((i2c->Instance == I2C1) ? "I2C1" : "I2C2"), devaddr, regaddr, i2c->ErrorCode);
	}
	return status;
}

