/*
 * DWT_Delay.h
 */
#ifndef _DWT_DELAY_H_
#define _DWT_DELAY_H_

#include "stm32f7xx_hal.h"

#define DWT_US_TUNING	1		// Tune value for overhead

uint32_t DWT_Delay_Init(void);

// Delay in microseconds using DWT
__STATIC_INLINE void DWT_Delay_us(volatile uint32_t delay_us)
{
	uint32_t start_ticks = DWT->CYCCNT;
	uint32_t DWT_us_ticks = (HAL_RCC_GetHCLKFreq() / 1000000);
	delay_us = delay_us * DWT_us_ticks - DWT_US_TUNING;
	while ((DWT->CYCCNT - start_ticks) < delay_us);
}

// Delay in milliseconds using DWT
__STATIC_INLINE void DWT_Delay_ms(volatile uint32_t delay_ms)
{
	uint32_t start_ticks = DWT->CYCCNT;
    uint32_t DWT_ms_ticks = (HAL_RCC_GetHCLKFreq() / 1000);
	delay_ms *= DWT_ms_ticks;
	while ((DWT->CYCCNT - start_ticks) < delay_ms);
}

// Get DWT ticks
__STATIC_INLINE uint32_t DWT_GetTick()
{
	return DWT->CYCCNT / (HAL_RCC_GetHCLKFreq() / 1000000);
}

#endif /* _DWT_DELAY_H_ */
