/**
  ******************************************************************************
  * @file    uac2.h
  * @author  MK
  * @version V0.1
  * @date    08/01/2021
  * @brief   USD Audio Class 2 descriptors.
  ******************************************************************************
  */
#ifndef __UAC2_H__
#define __UAC2_H__

#ifdef __cplusplus
extern "C" {
#endif

#define ATTR_PACKED __attribute__ ((packed))

// Audio Device Class Codes

// A.1 - Audio Functioon Class Codes
typedef enum
{
  UAC2_FUNCTION_CLASS_AUDIO = 0x01,
} uac2_function_class_type_t;

// A.2 - Audio Function Subclass Codes
typedef enum
{
  UAC2_FUNCTION_SUBCLASS_UNDEFINED = 0x00,
} uac2_function_subclass_type_t;

// A.3 - Audio Function Protocol Codes
typedef enum
{
  UAC2_FUNC_PROTOCOL_CODE_UNDEF       = 0x00,
  UAC2_FUNC_PROTOCOL_CODE_V2          = 0x20, // Version 2.0
} uac2_function_protocol_code_t;

// A.4 - Audio Interface Class Codes
typedef enum
{
  UAC2_INTERFACE_CLASS_AUDIO = 0x01,
} uac2_interface_class_type_t;

// A.5 - Audio Interface Subclass Codes
typedef enum
{
  UAC2_INTERFACE_SUBCLASS_UNDEFINED      = 0x00,
  UAC2_INTERFACE_SUBCLASS_CONTROL        = 0x01, // Audio Control
  UAC2_INTERFACE_SUBCLASS_STREAMING      = 0x02, // Audio Streaming
  UAC2_INTERFACE_SUBCLASS_MIDI_STREAMING = 0x03, // MIDI Streaming
} uac2_interface_subclass_type_t;

// A.6 - Audio Interface Protocol Codes
typedef enum
{
  UAC2_INTERFACE_PROTOCOL_CODE_UNDEF     = 0x00,
  UAC2_INTERFACE_PROTOCOL_CODE_V2        = 0x20, // Version 2.0
} uac2_interface_protocol_code_t;

// A.7 - Audio Function Category Codes
typedef enum
{
  UAC2_FUNC_UNDEF              = 0x00,
  UAC2_FUNC_DESKTOP_SPEAKER    = 0x01,
  UAC2_FUNC_HOME_THEATER       = 0x02,
  UAC2_FUNC_MICROPHONE         = 0x03,
  UAC2_FUNC_HEADSET            = 0x04,
  UAC2_FUNC_TELEPHONE          = 0x05,
  UAC2_FUNC_CONVERTER          = 0x06,
  UAC2_FUNC_SOUND_RECODER      = 0x07,
  UAC2_FUNC_IO_BOX             = 0x08,
  UAC2_FUNC_MUSICAL_INSTRUMENT = 0x09,
  UAC2_FUNC_PRO_AUDIO          = 0x0A,
  UAC2_FUNC_AUDIO_VIDEO        = 0x0B,
  UAC2_FUNC_CONTROL_PANEL      = 0x0C,
  UAC2_FUNC_OTHER              = 0xFF,
} uac2_function_code_t;

// A.8 - Audio Class-Specific Descriptor Types
typedef enum
{
  UAC2_CS_UNDEF                = 0x20,
  UAC2_CS_DEVICE               = 0x21,
  UAC2_CS_CONFIGURATION        = 0x22,
  UAC2_CS_STRING               = 0x23,
  UAC2_CS_INTERFACE            = 0x24,
  UAC2_CS_ENDPOINT             = 0x25,
} uac2_cs_descriptor_code_t;

// A.9 - Audio Class-Specific AC Interface Descriptor Subtypes
typedef enum
{
  UAC2_CS_AC_INTERFACE_AC_DESCRIPTOR_UNDEF   = 0x00,
  UAC2_CS_AC_INTERFACE_HEADER                = 0x01,
  UAC2_CS_AC_INTERFACE_INPUT_TERMINAL        = 0x02,
  UAC2_CS_AC_INTERFACE_OUTPUT_TERMINAL       = 0x03,
  UAC2_CS_AC_INTERFACE_MIXER_UNIT            = 0x04,
  UAC2_CS_AC_INTERFACE_SELECTOR_UNIT         = 0x05,
  UAC2_CS_AC_INTERFACE_FEATURE_UNIT          = 0x06,
  UAC2_CS_AC_INTERFACE_EFFECT_UNIT           = 0x07,
  UAC2_CS_AC_INTERFACE_PROCESSING_UNIT       = 0x08,
  UAC2_CS_AC_INTERFACE_EXTENSION_UNIT        = 0x09,
  UAC2_CS_AC_INTERFACE_CLOCK_SOURCE          = 0x0A,
  UAC2_CS_AC_INTERFACE_CLOCK_SELECTOR        = 0x0B,
  UAC2_CS_AC_INTERFACE_CLOCK_MULTIPLIER      = 0x0C,
  UAC2_CS_AC_INTERFACE_SAMPLE_RATE_CONVERTER = 0x0D,
} uac2_cs_ac_interface_subtype_t;

// A.10 - Audio Class-Specific AS Interface Descriptor Subtypes UAC2
typedef enum
{
  UAC2_CS_AS_INTERFACE_AS_DESCRIPTOR_UNDEF   = 0x00,
  UAC2_CS_AS_INTERFACE_AS_GENERAL            = 0x01,
  UAC2_CS_AS_INTERFACE_FORMAT_TYPE           = 0x02,
  UAC2_CS_AS_INTERFACE_ENCODER               = 0x03,
  UAC2_CS_AS_INTERFACE_DECODER               = 0x04,
} uac2_cs_as_interface_subtype_t;

// A.11 - Effect Unit Effect Types
typedef enum
{
  UAC2_EFFECT_TYPE_UNDEF                     = 0x00,
  UAC2_EFFECT_TYPE_PARAM_EQ_SECTION          = 0x01,
  UAC2_EFFECT_TYPE_REVERBERATION             = 0x02,
  UAC2_EFFECT_TYPE_MOD_DELAY                 = 0x03,
  UAC2_EFFECT_TYPE_DYN_RANGE_COMP            = 0x04,
} uac2_effect_unit_effect_type_t;

// A.12 - Processing Unit Process Types
typedef enum
{
  UAC2_PROCESS_TYPE_UNDEF                    = 0x00,
  UAC2_PROCESS_TYPE_UP_DOWN_MIX              = 0x01,
  UAC2_PROCESS_TYPE_DOLBY_PROLOGIC           = 0x02,
  UAC2_PROCESS_TYPE_STEREO_EXTENDER          = 0x03,
} uac2_processing_unit_process_type_t;

// A.13 - Audio Class-Specific EP Descriptor Subtypes UAC2
typedef enum
{
  UAC2_CS_EP_SUBTYPE_UNDEF                   = 0x00,
  UAC2_CS_EP_SUBTYPE_GENERAL                 = 0x01,
} uac2_cs_ep_subtype_t;

// A.14 - Audio Class-Specific Request Codes
typedef enum
{
  UAC2_CS_REQ_UNDEF                          = 0x00,
  UAC2_CS_REQ_CUR                            = 0x01,
  UAC2_CS_REQ_RANGE                          = 0x02,
  UAC2_CS_REQ_MEM                            = 0x03,
} uac2_cs_req_t;

// A.17 - Control Selector Codes

// A.17.1 - Clock Source Control Selectors
typedef enum
{
  UAC2_CS_CTRL_UNDEF                         = 0x00,
  UAC2_CS_CTRL_SAMPLING_FREQ                 = 0x01,
  UAC2_CS_CTRL_CLK_VALID                     = 0x02,
} uac2_clock_src_control_selector_t;

/// A.17.2 - Clock Selector Control Selectors
typedef enum
{
  UAC2_CX_CTRL_UNDEF                         = 0x00,
  UAC2_CX_CTRL_CONTROL                       = 0x01,
} UAC2_clock_sel_control_selector_t;

/// A.17.3 - Clock Multiplier Control Selectors
typedef enum
{
  UAC2_CM_CTRL_UNDEF                         = 0x00,
  UAC2_CM_CTRL_NUMERATOR_CONTROL             = 0x01,
  UAC2_CM_CTRL_DENOMINATOR_CONTROL           = 0x02,
} UAC2_clock_mul_control_selector_t;

/// A.17.4 - Terminal Control Selectors
typedef enum
{
  UAC2_TE_CTRL_UNDEF                         = 0x00,
  UAC2_TE_CTRL_COPY_PROTECT                  = 0x01,
  UAC2_TE_CTRL_CONNECTOR                     = 0x02,
  UAC2_TE_CTRL_OVERLOAD                      = 0x03,
  UAC2_TE_CTRL_CLUSTER                       = 0x04,
  UAC2_TE_CTRL_UNDERFLOW                     = 0x05,
  UAC2_TE_CTRL_OVERFLOW                      = 0x06,
  UAC2_TE_CTRL_LATENCY                       = 0x07,
} uac2_terminal_control_selector_t;

/// A.17.5 - Mixer Control Selectors
typedef enum
{
  UAC2_MU_CTRL_UNDEF                         = 0x00,
  UAC2_MU_CTRL_MIXER                         = 0x01,
  UAC2_MU_CTRL_CLUSTER                       = 0x02,
  UAC2_MU_CTRL_UNDERFLOW                     = 0x03,
  UAC2_MU_CTRL_OVERFLOW                      = 0x04,
  UAC2_MU_CTRL_LATENCY                       = 0x05,
} uac2_mixer_control_selector_t;

/// A.17.6 - Selector Control Selectors
typedef enum
{
  UAC2_SU_CTRL_UNDEF                         = 0x00,
  UAC2_SU_CTRL_SELECTOR                      = 0x01,
  UAC2_SU_CTRL_LATENCY                       = 0x02,
} uac2_sel_control_selector_t;

/// A.17.7 - Feature Unit Control Selectors
typedef enum
{
  UAC2_FU_CTRL_UNDEF                         = 0x00,
  UAC2_FU_CTRL_MUTE                          = 0x01,
  UAC2_FU_CTRL_VOLUME                        = 0x02,
  UAC2_FU_CTRL_BASS                          = 0x03,
  UAC2_FU_CTRL_MID                           = 0x04,
  UAC2_FU_CTRL_TREBLE                        = 0x05,
  UAC2_FU_CTRL_GRAPHIC_EQUALIZER             = 0x06,
  UAC2_FU_CTRL_AGC                           = 0x07,
  UAC2_FU_CTRL_DELAY                         = 0x08,
  UAC2_FU_CTRL_BASS_BOOST                    = 0x09,
  UAC2_FU_CTRL_LOUDNESS                      = 0x0A,
  UAC2_FU_CTRL_INPUT_GAIN                    = 0x0B,
  UAC2_FU_CTRL_GAIN_PAD                      = 0x0C,
  UAC2_FU_CTRL_INVERTER                      = 0x0D,
  UAC2_FU_CTRL_UNDERFLOW                     = 0x0E,
  UAC2_FU_CTRL_OVERVLOW                      = 0x0F,
  UAC2_FU_CTRL_LATENCY                       = 0x10,
} uac2_feature_unit_control_selector_t;

/// A.17.8 Effect Unit Control Selectors

/// A.17.8.1 Parametric Equalizer Section Effect Unit Control Selectors
typedef enum
{
  UAC2_PE_CTRL_UNDEF                         = 0x00,
  UAC2_PE_CTRL_ENABLE                        = 0x01,
  UAC2_PE_CTRL_CENTERFREQ                    = 0x02,
  UAC2_PE_CTRL_QFACTOR                       = 0x03,
  UAC2_PE_CTRL_GAIN                          = 0x04,
  UAC2_PE_CTRL_UNDERFLOW                     = 0x05,
  UAC2_PE_CTRL_OVERFLOW                      = 0x06,
  UAC2_PE_CTRL_LATENCY                       = 0x07,
} uac2_parametric_equalizer_control_selector_t;

/// A.17.8.2 Reverberation Effect Unit Control Selectors
typedef enum
{
  UAC2_RV_CTRL_UNDEF                         = 0x00,
  UAC2_RV_CTRL_ENABLE                        = 0x01,
  UAC2_RV_CTRL_TYPE                          = 0x02,
  UAC2_RV_CTRL_LEVEL                         = 0x03,
  UAC2_RV_CTRL_TIME                          = 0x04,
  UAC2_RV_CTRL_FEEDBACK                      = 0x05,
  UAC2_RV_CTRL_PREDELAY                      = 0x06,
  UAC2_RV_CTRL_DENSITY                       = 0x07,
  UAC2_RV_CTRL_HIFREQ_ROLLOFF                = 0x08,
  UAC2_RV_CTRL_UNDERFLOW                     = 0x09,
  UAC2_RV_CTRL_OVERFLOW                      = 0x0A,
  UAC2_RV_CTRL_LATENCY                       = 0x0B,
} uac2_reverberation_effect_control_selector_t;

/// A.17.8.3 Modulation Delay Effect Unit Control Selectors
typedef enum
{
  UAC2_MD_CTRL_UNDEF                         = 0x00,
  UAC2_MD_CTRL_ENABLE                        = 0x01,
  UAC2_MD_CTRL_BALANCE                       = 0x02,
  UAC2_MD_CTRL_RATE                          = 0x03,
  UAC2_MD_CTRL_DEPTH                         = 0x04,
  UAC2_MD_CTRL_TIME                          = 0x05,
  UAC2_MD_CTRL_FEEDBACK                      = 0x06,
  UAC2_MD_CTRL_UNDERFLOW                     = 0x07,
  UAC2_MD_CTRL_OVERFLOW                      = 0x08,
  UAC2_MD_CTRL_LATENCY                       = 0x09,
} uac2_modulation_delay_control_selector_t;

/// A.17.8.4 Dynamic Range Compressor Effect Unit Control Selectors
typedef enum
{
  UAC2_DR_CTRL_UNDEF                         = 0x00,
  UAC2_DR_CTRL_ENABLE                        = 0x01,
  UAC2_DR_CTRL_COMPRESSION_RATE              = 0x02,
  UAC2_DR_CTRL_MAXAMPL                       = 0x03,
  UAC2_DR_CTRL_THRESHOLD                     = 0x04,
  UAC2_DR_CTRL_ATTACK_TIME                   = 0x05,
  UAC2_DR_CTRL_RELEASE_TIME                  = 0x06,
  UAC2_DR_CTRL_UNDERFLOW                     = 0x07,
  UAC2_DR_CTRL_OVERFLOW                      = 0x08,
  UAC2_DR_CTRL_LATENCY                       = 0x09,
} uac2_dynamic_range_compression_control_selector_t;

/// A.17.9 Processing Unit Control Selectors

/// A.17.9.1 Up/Down-mix Processing Unit Control Selectors
typedef enum
{
  UAC2_UD_CTRL_UNDEF                         = 0x00,
  UAC2_UD_CTRL_ENABLE                        = 0x01,
  UAC2_UD_CTRL_MODE_SELECT                   = 0x02,
  UAC2_UD_CTRL_CLUSTER                       = 0x03,
  UAC2_UD_CTRL_UNDERFLOW                     = 0x04,
  UAC2_UD_CTRL_OVERFLOW                      = 0x05,
  UAC2_UD_CTRL_LATENCY                       = 0x06,
} uac2_up_down_mix_control_selector_t;

/// A.17.9.2 Dolby Prologic ™ Processing Unit Control Selectors
typedef enum
{
  UAC2_DP_CTRL_UNDEF                         = 0x00,
  UAC2_DP_CTRL_ENABLE                        = 0x01,
  UAC2_DP_CTRL_MODE_SELECT                   = 0x02,
  UAC2_DP_CTRL_CLUSTER                       = 0x03,
  UAC2_DP_CTRL_UNDERFLOW                     = 0x04,
  UAC2_DP_CTRL_OVERFLOW                      = 0x05,
  UAC2_DP_CTRL_LATENCY                       = 0x06,
} uac2_dolby_prologic_control_selector_t;

/// A.17.9.3 Stereo Extender Processing Unit Control Selectors
typedef enum
{
  UAC2_ST_EXT_CTRL_UNDEF                     = 0x00,
  UAC2_ST_EXT_CTRL_ENABLE                    = 0x01,
  UAC2_ST_EXT_CTRL_WIDTH                     = 0x02,
  UAC2_ST_EXT_CTRL_UNDERFLOW                 = 0x03,
  UAC2_ST_EXT_CTRL_OVERFLOW                  = 0x04,
  UAC2_ST_EXT_CTRL_LATENCY                   = 0x05,
} uac2_stereo_extender_control_selector_t;

/// A.17.10 Extension Unit Control Selectors
typedef enum
{
  UAC2_XU_CTRL_UNDEF                         = 0x00,
  UAC2_XU_CTRL_ENABLE                        = 0x01,
  UAC2_XU_CTRL_CLUSTER                       = 0x02,
  UAC2_XU_CTRL_UNDERFLOW                     = 0x03,
  UAC2_XU_CTRL_OVERFLOW                      = 0x04,
  UAC2_XU_CTRL_LATENCY                       = 0x05,
} uac2_extension_unit_control_selector_t;

/// A.17.11 AudioStreaming Interface Control Selectors
typedef enum
{
  UAC2_AS_CTRL_UNDEF                         = 0x00,
  UAC2_AS_CTRL_ACT_ALT_SETTING               = 0x01,
  UAC2_AS_CTRL_VAL_ALT_SETTINGS              = 0x02,
  UAC2_AS_CTRL_AUDIO_DATA_FORMAT             = 0x03,
} uac2_audiostreaming_interface_control_selector_t;

/// A.17.12 Encoder Control Selectors
typedef enum
{
  UAC2_EN_CTRL_UNDEF                         = 0x00,
  UAC2_EN_CTRL_BIT_RATE                      = 0x01,
  UAC2_EN_CTRL_QUALITY                       = 0x02,
  UAC2_EN_CTRL_VBR                           = 0x03,
  UAC2_EN_CTRL_TYPE                          = 0x04,
  UAC2_EN_CTRL_UNDERFLOW                     = 0x05,
  UAC2_EN_CTRL_OVERFLOW                      = 0x06,
  UAC2_EN_CTRL_ENCODER_ERROR                 = 0x07,
  UAC2_EN_CTRL_PARAM1                        = 0x08,
  UAC2_EN_CTRL_PARAM2                        = 0x09,
  UAC2_EN_CTRL_PARAM3                        = 0x0A,
  UAC2_EN_CTRL_PARAM4                        = 0x0B,
  UAC2_EN_CTRL_PARAM5                        = 0x0C,
  UAC2_EN_CTRL_PARAM6                        = 0x0D,
  UAC2_EN_CTRL_PARAM7                        = 0x0E,
  UAC2_EN_CTRL_PARAM8                        = 0x0F,
} uac2_encoder_control_selector_t;

/// A.17.13 Decoder Control Selectors

/// A.17.13.1 MPEG Decoder Control Selectors
typedef enum
{
  UAC2_MPD_CTRL_UNDEF                        = 0x00,
  UAC2_MPD_CTRL_DUAL_CHANNEL                 = 0x01,
  UAC2_MPD_CTRL_SECOND_STEREO                = 0x02,
  UAC2_MPD_CTRL_MULTILINGUAL                 = 0x03,
  UAC2_MPD_CTRL_DYN_RANGE                    = 0x04,
  UAC2_MPD_CTRL_SCALING                      = 0x05,
  UAC2_MPD_CTRL_HILO_SCALING                 = 0x06,
  UAC2_MPD_CTRL_UNDERFLOW                    = 0x07,
  UAC2_MPD_CTRL_OVERFLOW                     = 0x08,
  UAC2_MPD_CTRL_DECODER_ERROR                = 0x09,
} uac2_MPEG_decoder_control_selector_t;

/// A.17.13.2 AC-3 Decoder Control Selectors
typedef enum
{
  UAC2_AD_CTRL_UNDEF                         = 0x00,
  UAC2_AD_CTRL_MODE                          = 0x01,
  UAC2_AD_CTRL_DYN_RANGE                     = 0x02,
  UAC2_AD_CTRL_SCALING                       = 0x03,
  UAC2_AD_CTRL_HILO_SCALING                  = 0x04,
  UAC2_AD_CTRL_UNDERFLOW                     = 0x05,
  UAC2_AD_CTRL_OVERFLOW                      = 0x06,
  UAC2_AD_CTRL_DECODER_ERROR                 = 0x07,
} uac2_AC3_decoder_control_selector_t;

/// A.17.13.3 WMA Decoder Control Selectors
typedef enum
{
  UAC2_WD_CTRL_UNDEF                         = 0x00,
  UAC2_WD_CTRL_UNDERFLOW                     = 0x01,
  UAC2_WD_CTRL_OVERFLOW                      = 0x02,
  UAC2_WD_CTRL_DECODER_ERROR                 = 0x03,
} uac2_WMA_decoder_control_selector_t;

/// A.17.13.4 DTS Decoder Control Selectors
typedef enum
{
  UAC2_DD_CTRL_UNDEF                         = 0x00,
  UAC2_DD_CTRL_UNDERFLOW                     = 0x01,
  UAC2_DD_CTRL_OVERFLOW                      = 0x02,
  UAC2_DD_CTRL_DECODER_ERROR                 = 0x03,
} uac2_DTS_decoder_control_selector_t;

/// A.17.14 Endpoint Control Selectors
typedef enum
{
  UAC2_EP_CTRL_UNDEF                         = 0x00,
  UAC2_EP_CTRL_PITCH                         = 0x01,
  UAC2_EP_CTRL_DATA_OVERRUN                  = 0x02,
  UAC2_EP_CTRL_DATA_UNDERRUN                 = 0x03,
} uac2_EP_control_selector_t;

/// Terminal Types

/// 2.1 - Audio Class-Terminal Types UAC2
typedef enum
{
  UAC2_TERM_TYPE_USB_UNDEFINED       = 0x0100,
  UAC2_TERM_TYPE_USB_STREAMING       = 0x0101,
  UAC2_TERM_TYPE_USB_VENDOR_SPEC     = 0x01FF,
} uac2_terminal_type_t;

/// 2.2 - Audio Class-Input Terminal Types UAC2
typedef enum
{
  UAC2_TERM_TYPE_IN_UNDEFINED        = 0x0200,
  UAC2_TERM_TYPE_IN_GENERIC_MIC      = 0x0201,
  UAC2_TERM_TYPE_IN_DESKTOP_MIC      = 0x0202,
  UAC2_TERM_TYPE_IN_PERSONAL_MIC     = 0x0203,
  UAC2_TERM_TYPE_IN_OMNI_MIC         = 0x0204,
  UAC2_TERM_TYPE_IN_ARRAY_MIC        = 0x0205,
  UAC2_TERM_TYPE_IN_PROC_ARRAY_MIC   = 0x0206,
} uac2_terminal_input_type_t;

/// 2.3 - Audio Class-Output Terminal Types UAC2
typedef enum
{
  UAC2_TERM_TYPE_OUT_UNDEFINED               = 0x0300,
  UAC2_TERM_TYPE_OUT_GENERIC_SPEAKER         = 0x0301,
  UAC2_TERM_TYPE_OUT_HEADPHONES              = 0x0302,
  UAC2_TERM_TYPE_OUT_HEAD_MNT_DISP_AUIDO     = 0x0303,
  UAC2_TERM_TYPE_OUT_DESKTOP_SPEAKER         = 0x0304,
  UAC2_TERM_TYPE_OUT_ROOM_SPEAKER            = 0x0305,
  UAC2_TERM_TYPE_OUT_COMMUNICATION_SPEAKER   = 0x0306,
  UAC2_TERM_TYPE_OUT_LOW_FRQ_EFFECTS_SPEAKER = 0x0307,
} uac2_terminal_output_type_t;

/// 2.6 - External Terminal Types UAC2
typedef enum
{
  UAC2_TERM_TYPE_EXT_UNDEFINED                 = 0x0600,
  UAC2_TERM_TYPE_EXT_ANALOG_CONNECTOR          = 0x0601,
  UAC2_TERM_TYPE_EXT_DIGITAL_AUDIO_INTERFACE   = 0x0602,
  UAC2_TERM_TYPE_EXT_LINE_CONNECTOR            = 0x0603,
  UAC2_TERM_TYPE_EXT_LEGACY_AUDIO_CONNECTOR    = 0x0604,
  UAC2_TERM_TYPE_EXT_SPDIF_INTERFACE           = 0x0605,
  UAC2_TERM_TYPE_EXT_1394_DA_STREAM            = 0x0606,
  UAC2_TERM_TYPE_EXT_1394_DV_STREAM_SOUNDTRACK = 0x0607,
  UAC2_TERM_TYPE_EXT_ADAT_LIGHTPIPE            = 0x0608,
  UAC2_TERM_TYPE_EXT_TDIF                      = 0x0609,
  UAC2_TERM_TYPE_EXT_MADI                      = 0x060A,
} uac2_terminal_external_type_t;
/// Rest is yet to be implemented

/// Additional Audio Device Class Codes - Source: Audio Data Formats

/// A.1 - Audio Class-Format Type Codes UAC2
typedef enum
{
  UAC2_FORMAT_TYPE_UNDEFINED     = 0x00,
  UAC2_FORMAT_TYPE_I             = 0x01,
  UAC2_FORMAT_TYPE_II            = 0x02,
  UAC2_FORMAT_TYPE_III           = 0x03,
  UAC2_FORMAT_TYPE_IV            = 0x04,
  UAC2_EXT_FORMAT_TYPE_I         = 0x81,
  UAC2_EXT_FORMAT_TYPE_II        = 0x82,
  UAC2_EXT_FORMAT_TYPE_III       = 0x83,
} uac2_format_type_t;

// A.2.1 - Audio Class-Audio Data Format Type I UAC2
typedef enum
{
  UAC2_DATA_FORMAT_TYPE_I_PCM            = 0x00000001,
  UAC2_DATA_FORMAT_TYPE_I_PCM8           = 0x00000002,
  UAC2_DATA_FORMAT_TYPE_I_IEEE_FLOAT     = 0x00000004,
  UAC2_DATA_FORMAT_TYPE_I_ALAW           = 0x00000008,
  UAC2_DATA_FORMAT_TYPE_I_MULAW          = 0x00000010,
  UAC2_DATA_FORMAT_TYPE_I_RAW_DATA       = 0x80000000,
} uac2_data_format_type_I_t;

/// All remaining definitions are taken from the descriptor descriptions in the UAC2 main specification

/// Isochronous End Point Attributes
typedef enum
{
  TUSB_ISO_EP_ATT_NO_SYNC         = 0x00,
  TUSB_ISO_EP_ATT_ASYNCHRONOUS    = 0x04,
  TUSB_ISO_EP_ATT_ADAPTIVE        = 0x08,
  TUSB_ISO_EP_ATT_SYNCHRONOUS     = 0x0C,
  TUSB_ISO_EP_ATT_DATA            = 0x00, ///< Data End Point
  TUSB_ISO_EP_ATT_EXPLICIT_FB     = 0x10, ///< Feedback End Point
  TUSB_ISO_EP_ATT_IMPLICIT_FB     = 0x20, ///< Data endpoint that also serves as an implicit feedback
} tusb_iso_ep_attribute_t;

/// Audio Class-Control Values UAC2
typedef enum
{
  UAC2_CTRL_NONE     = 0x00,         // No Host access
  UAC2_CTRL_R        = 0x01,         // Host read access only
  UAC2_CTRL_RW       = 0x03,         // Host read write access
} uac2_control_t;

/// Audio Class-Specific AC Interface Descriptor Controls UAC2
typedef enum
{
  UAC2_CS_AS_INTERFACE_CTRL_LATENCY_POS  = 0,
} uac2_cs_ac_interface_control_pos_t;

/// Audio Class-Specific AS Interface Descriptor Controls UAC2
typedef enum
{
  UAC2_CS_AS_INTERFACE_CTRL_ACTIVE_ALT_SET_POS   = 0,
  UAC2_CS_AS_INTERFACE_CTRL_VALID_ALT_SET_POS    = 2,
} uac2_cs_as_interface_control_pos_t;

/// Audio Class-Specific AS Isochronous Data EP Attributes UAC2
typedef enum
{
  UAC2_CS_AS_ISO_DATA_EP_ATT_MAX_PACKETS_ONLY    = 0x80,
  UAC2_CS_AS_ISO_DATA_EP_ATT_NON_MAX_PACKETS_OK  = 0x00,
} uac2_cs_as_iso_data_ep_attribute_t;

/// Audio Class-Specific AS Isochronous Data EP Controls UAC2
typedef enum
{
  UAC2_CS_AS_ISO_DATA_EP_CTRL_PITCH_POS          = 0,
  UAC2_CS_AS_ISO_DATA_EP_CTRL_DATA_OVERRUN_POS   = 2,
  UAC2_CS_AS_ISO_DATA_EP_CTRL_DATA_UNDERRUN_POS  = 4,
} uac2_cs_as_iso_data_ep_control_pos_t;

/// Audio Class-Specific AS Isochronous Data EP Lock Delay Units UAC2
typedef enum
{
  UAC2_CS_AS_ISO_DATA_EP_LOCK_DELAY_UNIT_UNDEFINED       = 0x00,
  UAC2_CS_AS_ISO_DATA_EP_LOCK_DELAY_UNIT_MILLISEC        = 0x01,
  UAC2_CS_AS_ISO_DATA_EP_LOCK_DELAY_UNIT_PCM_SAMPLES     = 0x02,
} uac2_cs_as_iso_data_ep_lock_delay_unit_t;

/// Audio Class-Clock Source Attributes UAC2
typedef enum
{
  UAC2_CLOCK_SOURCE_ATT_EXT_CLK      = 0x00,
  UAC2_CLOCK_SOURCE_ATT_INT_FIX_CLK  = 0x01,
  UAC2_CLOCK_SOURCE_ATT_INT_VAR_CLK  = 0x02,
  UAC2_CLOCK_SOURCE_ATT_INT_PRO_CLK  = 0x03,
  UAC2_CLOCK_SOURCE_ATT_CLK_SYC_SOF  = 0x04,
} uac2_clock_source_attribute_t;

/// Audio Class-Clock Source Controls UAC2
typedef enum
{
  UAC2_CLOCK_SOURCE_CTRL_CLK_FRQ_POS     = 0,
  UAC2_CLOCK_SOURCE_CTRL_CLK_VAL_POS     = 2,
} uac2_clock_source_control_pos_t;

/// Audio Class-Clock Selector Controls UAC2
typedef enum
{
  UAC2_CLOCK_SELECTOR_CTRL_POS   = 0,
} uac2_clock_selector_control_pos_t;

/// Audio Class-Clock Multiplier Controls UAC2
typedef enum
{
  UAC2_CLOCK_MULTIPLIER_CTRL_NUMERATOR_POS       = 0,
  UAC2_CLOCK_MULTIPLIER_CTRL_DENOMINATOR_POS     = 2,
} uac2_clock_multiplier_control_pos_t;

/// Audio Class-Input Terminal Controls UAC2
typedef enum
{
  UAC2_IN_TERM_CTRL_CPY_PROT_POS     = 0,
  UAC2_IN_TERM_CTRL_CONNECTOR_POS    = 2,
  UAC2_IN_TERM_CTRL_OVERLOAD_POS     = 4,
  UAC2_IN_TERM_CTRL_CLUSTER_POS      = 6,
  UAC2_IN_TERM_CTRL_UNDERFLOW_POS    = 8,
  UAC2_IN_TERM_CTRL_OVERFLOW_POS     = 10,
} uac2_terminal_input_control_pos_t;

/// Audio Class-Output Terminal Controls UAC2
typedef enum
{
  UAC2_OUT_TERM_CTRL_CPY_PROT_POS    = 0,
  UAC2_OUT_TERM_CTRL_CONNECTOR_POS   = 2,
  UAC2_OUT_TERM_CTRL_OVERLOAD_POS    = 4,
  UAC2_OUT_TERM_CTRL_UNDERFLOW_POS   = 6,
  UAC2_OUT_TERM_CTRL_OVERFLOW_POS    = 8,
} uac2_terminal_output_control_pos_t;

/// Audio Class-Feature Unit Controls UAC2
typedef enum
{
  UAC2_FEATURE_UNIT_CTRL_MUTE_POS            = 0,
  UAC2_FEATURE_UNIT_CTRL_VOLUME_POS          = 2,
  UAC2_FEATURE_UNIT_CTRL_BASS_POS            = 4,
  UAC2_FEATURE_UNIT_CTRL_MID_POS             = 6,
  UAC2_FEATURE_UNIT_CTRL_TREBLE_POS          = 8,
  UAC2_FEATURE_UNIT_CTRL_GRAPHIC_EQU_POS     = 10,
  UAC2_FEATURE_UNIT_CTRL_AGC_POS             = 12,
  UAC2_FEATURE_UNIT_CTRL_DELAY_POS           = 14,
  UAC2_FEATURE_UNIT_CTRL_BASS_BOOST_POS      = 16,
  UAC2_FEATURE_UNIT_CTRL_LOUDNESS_POS        = 18,
  UAC2_FEATURE_UNIT_CTRL_INPUT_GAIN_POS      = 20,
  UAC2_FEATURE_UNIT_CTRL_INPUT_GAIN_PAD_POS  = 22,
  UAC2_FEATURE_UNIT_CTRL_PHASE_INV_POS       = 24,
  UAC2_FEATURE_UNIT_CTRL_UNDERFLOW_POS       = 26,
  UAC2_FEATURE_UNIT_CTRL_OVERFLOW_POS        = 28,
} uac2_feature_unit_control_pos_t;

/// Audio Class-Audio Channel Configuration UAC2
typedef enum
{
  UAC2_CHANNEL_CONFIG_NON_PREDEFINED             = 0x00000000,
  UAC2_CHANNEL_CONFIG_FRONT_LEFT                 = 0x00000001,
  UAC2_CHANNEL_CONFIG_FRONT_RIGHT                = 0x00000002,
  UAC2_CHANNEL_CONFIG_FRONT_CENTER               = 0x00000004,
  UAC2_CHANNEL_CONFIG_LOW_FRQ_EFFECTS            = 0x00000008,
  UAC2_CHANNEL_CONFIG_BACK_LEFT                  = 0x00000010,
  UAC2_CHANNEL_CONFIG_BACK_RIGHT                 = 0x00000020,
  UAC2_CHANNEL_CONFIG_FRONT_LEFT_OF_CENTER       = 0x00000040,
  UAC2_CHANNEL_CONFIG_FRONT_RIGHT_OF_CENTER      = 0x00000080,
  UAC2_CHANNEL_CONFIG_BACK_CENTER                = 0x00000100,
  UAC2_CHANNEL_CONFIG_SIDE_LEFT                  = 0x00000200,
  UAC2_CHANNEL_CONFIG_SIDE_RIGHT                 = 0x00000400,
  UAC2_CHANNEL_CONFIG_TOP_CENTER                 = 0x00000800,
  UAC2_CHANNEL_CONFIG_TOP_FRONT_LEFT             = 0x00001000,
  UAC2_CHANNEL_CONFIG_TOP_FRONT_CENTER           = 0x00002000,
  UAC2_CHANNEL_CONFIG_TOP_FRONT_RIGHT            = 0x00004000,
  UAC2_CHANNEL_CONFIG_TOP_BACK_LEFT              = 0x00008000,
  UAC2_CHANNEL_CONFIG_TOP_BACK_CENTER            = 0x00010000,
  UAC2_CHANNEL_CONFIG_TOP_BACK_RIGHT             = 0x00020000,
  UAC2_CHANNEL_CONFIG_TOP_FRONT_LEFT_OF_CENTER   = 0x00040000,
  UAC2_CHANNEL_CONFIG_TOP_FRONT_RIGHT_OF_CENTER  = 0x00080000,
  UAC2_CHANNEL_CONFIG_LEFT_LOW_FRQ_EFFECTS       = 0x00100000,
  UAC2_CHANNEL_CONFIG_RIGHT_LOW_FRQ_EFFECTS      = 0x00200000,
  UAC2_CHANNEL_CONFIG_TOP_SIDE_LEFT              = 0x00400000,
  UAC2_CHANNEL_CONFIG_TOP_SIDE_RIGHT             = 0x00800000,
  UAC2_CHANNEL_CONFIG_BOTTOM_CENTER              = 0x01000000,
  UAC2_CHANNEL_CONFIG_BACK_LEFT_OF_CENTER        = 0x02000000,
  UAC2_CHANNEL_CONFIG_BACK_RIGHT_OF_CENTER       = 0x04000000,
  UAC2_CHANNEL_CONFIG_RAW_DATA                   = 0x80000000,
} uac2_channel_config_t;

// 5.2.3 Control Request Parameter Block Layout

// 5.2.3.1 1-byte Control CUR Parameter Block
typedef struct ATTR_PACKED
{
  int8_t bCur               ;   // The setting for the CUR attribute of the addressed Control
} audio_control_cur_1_t;

// 5.2.3.2 2-byte Control CUR Parameter Block
typedef struct ATTR_PACKED
{
  int16_t bCur              ;   // The setting for the CUR attribute of the addressed Control
} audio_control_cur_2_t;

// 5.2.3.3 4-byte Control CUR Parameter Block
typedef struct ATTR_PACKED
{
  int32_t bCur              ;   // The setting for the CUR attribute of the addressed Control
} audio_control_cur_4_t;

// Use the following ONLY for RECEIVED data - compiler does not know how many subranges are defined! Use the one below for predefined lengths - or if you know what you are doing do what you like
// 5.2.3.1 1-byte Control RANGE Parameter Block
typedef struct ATTR_PACKED {
  uint16_t wNumSubRanges;
  struct ATTR_PACKED {
    int8_t bMin             ; /*The setting for the MIN attribute of the nth subrange of the addressed Control*/
    int8_t bMax             ; /*The setting for the MAX attribute of the nth subrange of the addressed Control*/
    uint8_t bRes            ; /*The setting for the RES attribute of the nth subrange of the addressed Control*/
  } subrange[]              ;
} audio_control_range_1_t;

// 5.2.3.2 2-byte Control RANGE Parameter Block
typedef struct ATTR_PACKED {
  uint16_t wNumSubRanges;
  struct ATTR_PACKED {
    int16_t bMin            ; /*The setting for the MIN attribute of the nth subrange of the addressed Control*/
    int16_t bMax            ; /*The setting for the MAX attribute of the nth subrange of the addressed Control*/
    uint16_t bRes           ; /*The setting for the RES attribute of the nth subrange of the addressed Control*/
  } subrange[]              ;
} audio_control_range_2_t;

// 5.2.3.3 4-byte Control RANGE Parameter Block
typedef struct ATTR_PACKED {
  uint16_t wNumSubRanges;
  struct ATTR_PACKED {
    int32_t bMin            ; /*The setting for the MIN attribute of the nth subrange of the addressed Control*/
    int32_t bMax            ; /*The setting for the MAX attribute of the nth subrange of the addressed Control*/
    uint32_t bRes           ; /*The setting for the RES attribute of the nth subrange of the addressed Control*/
  } subrange[]              ;
} audio_control_range_4_t;

// 5.2.3.1 1-byte Control RANGE Parameter Block
#define audio_control_range_1_n_t(numSubRanges) \
    struct ATTR_PACKED {                     \
  uint16_t wNumSubRanges;                       \
  struct ATTR_PACKED {                       \
      int8_t bMin               ; /*The setting for the MIN attribute of the nth subrange of the addressed Control*/\
    int8_t bMax                 ; /*The setting for the MAX attribute of the nth subrange of the addressed Control*/\
    uint8_t bRes                ; /*The setting for the RES attribute of the nth subrange of the addressed Control*/\
    } subrange[numSubRanges]    ;               \
}

/// 5.2.3.2 2-byte Control RANGE Parameter Block
#define audio_control_range_2_n_t(numSubRanges) \
    struct ATTR_PACKED {                     \
  uint16_t wNumSubRanges;                       \
  struct ATTR_PACKED {                       \
      int16_t bMin          ; /*The setting for the MIN attribute of the nth subrange of the addressed Control*/\
    int16_t bMax            ; /*The setting for the MAX attribute of the nth subrange of the addressed Control*/\
    uint16_t bRes           ; /*The setting for the RES attribute of the nth subrange of the addressed Control*/\
    } subrange[numSubRanges];                   \
}

// 5.2.3.3 4-byte Control RANGE Parameter Block
#define audio_control_range_4_n_t(numSubRanges) \
    struct ATTR_PACKED {                     \
  uint16_t wNumSubRanges;                       \
  struct ATTR_PACKED {                       \
      int32_t bMin          ; /*The setting for the MIN attribute of the nth subrange of the addressed Control*/\
    int32_t bMax            ; /*The setting for the MAX attribute of the nth subrange of the addressed Control*/\
    uint32_t bRes           ; /*The setting for the RES attribute of the nth subrange of the addressed Control*/\
    } subrange[numSubRanges];                   \
}


#ifdef __cplusplus
}
#endif

#endif
