/* Kernel includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

#include <stdio.h>
#include <string.h>
#include "config.h"
#include "audio_in_task.h"
#include "adc.h"
#include "queue_cmd.h"

#define AUDIO_IN_QUEUE_LENGTH		( 100 )
#define	AUDIO_IN_TASK_PRIORITY		(tskIDLE_PRIORITY + 1L)
#define AUDIO_IN_TASK_STACK_SIZE	configMINIMAL_STACK_SIZE
#define AUDIO_IN_TASK_PERIOD		portMAX_DELAY

static void processQueueCmd(QUEUE_MsgTypeDef msg);

void audioInLoop(void)
{
	QUEUE_MsgTypeDef msg;

	ADC_Init();
	while(1) {
		if (xQueueReceive( audio_in_queue, &msg, AUDIO_IN_TASK_PERIOD )) {
			processQueueCmd(msg);
		}
	}
}

static void audioInTask(void* pvParameters)
{
	audioInLoop();
}

void vStartAudioInTask( void )
{
	audio_in_queue = xQueueCreate( AUDIO_IN_QUEUE_LENGTH, sizeof( QUEUE_MsgTypeDef ) );
	if (audio_in_queue != NULL) {
		xTaskCreate(audioInTask, "AudioIn", AUDIO_IN_TASK_STACK_SIZE, NULL, AUDIO_IN_TASK_PRIORITY, NULL);
	}
}

static void processQueueCmd(QUEUE_MsgTypeDef msg)
{
	switch (msg.cmd) {
	case INIT:
		if (_cfg.debug) {
			printf("ADC Init\r\n");
		}
		ADC_Init();
		break;
	case FS:
		if (_cfg.debug) {
			printf("Set fs %ld\r\n", *(uint32_t *)&msg.data[0]);
		}
		ADC_SetFS(*(uint32_t *)&msg.data[0]);
		break;
	case MONO:
		if (_cfg.debug) {
			printf("Set mono %d\r\n", *(uint8_t *)&msg.data[0]);
		}
		ADC_SetMono(msg.data[0]);
		break;
	case RECORD:
		if (_cfg.debug) {
			printf("Start recording\r\n");
		}
		ADC_Record();
		break;
	default:
		break;
	}
}
