/**
  ******************************************************************************
  * File Name          : config.c
  * Description        : Configuration data
  ******************************************************************************
  */
#include "main.h"
#include <stdio.h>
#include <string.h>

CFG_TypeDef _cfg;

char *getFWVersion(void)
{
	return "R1.0" " " __DATE__ " " __TIME__;
}

char *getClkFrequency(int16_t clk_frequency)
{
	char *s = "11M_12M";
	switch (clk_frequency) {
	case CLK_22M_24M:
		s = "22M_24M";
		break;
	case CLK_45M_49M:
		s = "45M_49M";
		break;
	}
	return s;
}

char *getADC(void)
{
	char *s = "NONE";
	switch (_cfg.adc) {
	case ADC_AK5394:
		s = "AK5394";
		break;
	case ADC_ES9822PRO:
		s = "ES9822PRO";
		break;
	default:
		break;
	}
	return s;
}

CLK_FREQ_TypeDef getADCClkFreqFromIO()
{
	if (HAL_GPIO_ReadPin(IO_1_PORT, IO_1_PIN)) {
		return CLK_45M_49M;
	}
	else {
		return CLK_22M_24M;
	}
}

I2S_ADC_TypeDef getADCFromIO()
{
	if (HAL_GPIO_ReadPin(IO_0_PORT, IO_0_PIN)) {
		return ADC_ES9822PRO;
	}
	else {
		return ADC_AK5394;
	}
}

/**
  * @brief  Initializes the configuration data
  * @param  None
  * @retval None
  */
void CFG_Init(void)
{
	_cfg.uac1 = CFG_UAC1;
	_cfg.uac2 = CFG_UAC2;
	_cfg.i2s_out = CFG_I2S_OUT;
	_cfg.i2s_in = CFG_I2S_IN;
	_cfg.debug = CFG_DEBUG;
	_cfg.adc = getADCFromIO();
	_cfg.adc_clk_frequency = getADCClkFreqFromIO();
	_cfg.revision = CFG_REVISION;
	printf("\r\nVersion: %d/%s\r\n", _cfg.revision, getFWVersion());
	printf("ADC: %s\r\n", getADC());
	if (_cfg.adc == ADC_ES9822PRO) {
		printf("ADC Clk frequency: %s\r\n", getClkFrequency(_cfg.adc_clk_frequency));
	}
}
